
#include <qlcdnumber.h>
#include <qpushbutton.h>
#include <qregexp.h>
#include <qlabel.h>
#include <qcursor.h>

#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kmessagebox.h>

#include <kcombobox.h>
#include <ktextedit.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kiconloader.h>

#include <kprocess.h>
#include <kprocio.h>

#include <kdebug.h>

#include <kconfigdialog.h>

#include "ko2smsresult.h"
#include "sendsmsdialogimp.h"
#include "ko2smsapplet.h"
#include "ko2smsappletsettings.h"
#include "ko2smsappletsettingswidget.h"

ko2smsApplet::ko2smsApplet(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
    : KPanelApplet(configFile, type, actions, parent, name)
{
    // Get the current application configuration handle
    // ksConfig = config();
    
    readConfig();
    
    QPushButton *sms = new QPushButton( this );
    
    kdDebug() << UserIcon( "ko2smsapplet" ) << endl;
    sms->setPixmap( UserIcon( "ko2smsapplet" ) );
        
    sms->setMaximumSize( QSize( 32, 32 ) );
    QObject::connect( sms, SIGNAL( clicked() ), this, SLOT( smsDialog() ) );
    
    mainView = sms;
    mainView->show();
}


ko2smsApplet::~ko2smsApplet()
{
}

void ko2smsApplet::smsDialog()
{
    SendSMSDialogImp *smsD = new SendSMSDialogImp(0);
    smsD->parseConfigFile( configFile );
    
    if( smsD->exec() == QDialog::Accepted )
    {
        KProcIO *proc = new KProcIO;
        //proc->setUseShell( true, "/usr/bin/perl" );
        proc->setComm( KProcess::All );
        proc->enableReadSignals( true );
        
        *proc << scriptFile;
        *proc << "-message" << smsD->kte_msg->text();
        *proc << smsD->kcb_dest->currentText();
        //kdDebug() << "To: " << smsD->kcb_dest->currentText() << endl;
        
        QObject::connect( proc, SIGNAL(readReady(KProcIO *)),
                          this, SLOT( o2smsReadReady( KProcIO* ) ) );
        QObject::connect( proc, SIGNAL(processExited(KProcess *)),
                          this, SLOT( o2smsExited( KProcess * ) ) );
        
        o2smsResult = new KO2smsResult( 0 );
        o2smsResult->show();
        
        if( proc->start( KProcess::NotifyOnExit, true ) )
        {
            kdDebug() << "Started process..." << endl;
        }
        else
        {
            o2smsResult->te_results->setText( "Invalid script file configured" );
            o2smsResult->buttonOK->setEnabled( true );
            o2smsResult->setCursor( QCursor( Qt::ArrowCursor ) );
            kdDebug() << "Error starting process..." << endl;
        }
                
    }
}

void ko2smsApplet::o2smsExited( KProcess *proc )
{
    o2smsResult->buttonOK->setEnabled( true );
    o2smsResult->setCursor( QCursor( Qt::ArrowCursor ) );
}

void ko2smsApplet::o2smsReadReady( KProcIO *proc )
{
    kdDebug() << "ReadReady received" << endl;
    
    QString line;
    bool partial;
    
    while( proc->readln( line, true, &partial ) != -1 )
    {
            o2smsResult->te_results->append( line );
            kdDebug() << "Read    : " << line << endl;
    }
}

void ko2smsApplet::about()
{

    KAboutData *aboutData = new KAboutData( "ko2smsapplet",
        I18N_NOOP("KO2smsApplet"), "0.1", I18N_NOOP(
        "This is a frontend to the o2sms perl script by mackers "
        "(www.mackers.com): <i>\"o2sms is a grand script to send "
        "SMS's (or text messages) via the UNIX command line. Despite "
        "its name, it supports Vodafone (Ireland) and Meteor as well "
        "as o2 (Ireland) users.</i> "
        "ko2smsapplet supplies a quick and easy way to use this "
        "script for KDE users.<p>"),
        KAboutData::License_GPL, "(c) 2005, Barry O'Donovan");

    aboutData->addAuthor("Barry O'Donovan", 0, "barry.odonovan@kdemail.net", "http://www.barryodonovan.com" );
    aboutData->setHomepage( "http://www.barryodonovan.com/development/kde/ko2smsapplet" );
    aboutData->setBugAddress( "ko2smsapplet@barryodonovan.com" );
    

    KAboutApplication aboutDialog( aboutData );
    aboutDialog.exec();
}


void ko2smsApplet::help()
{
    KMessageBox::information(0, i18n("Ensure you set the correct (and full/absolute) paths\n" 
            "to your o2sms binary and configuration file through\n"
            "the \"Configure ko2smsapplet...\" menu option.")
    );
}


void ko2smsApplet::preferences()
{
    if ( KConfigDialog::showDialog( "settings" ) ) 
        return; 
    
    KConfigDialog* configDialog = new KConfigDialog( this, "settings", 
            ko2smsAppletSettings::self() ); 
    
    ko2smsAppletSettingsWidget *settingsWidget = 
            new ko2smsAppletSettingsWidget( 0, "KO2smsApplet Settings" );
    
    configDialog->addPage( settingsWidget, i18n("ko2smsApplet"), "ko2smsapplet" ); 
 
    connect( configDialog, SIGNAL( settingsChanged() ), 
             this, SLOT( updateConfiguration() ) ); 
 
    configDialog->show();
}

int ko2smsApplet::widthForHeight(int height) const
{
    return 32;
}

int ko2smsApplet::heightForWidth(int width) const
{
    return 32;
}

void ko2smsApplet::resizeEvent(QResizeEvent *e)
{
}


void ko2smsApplet::readConfig() 
{ 
    scriptFile = ko2smsAppletSettings::scriptFile(); 
    configFile = ko2smsAppletSettings::configFile(); 
} 

void ko2smsApplet::saveSettings() 
{ 
    ko2smsAppletSettings::setScriptFile( scriptFile ); 
    ko2smsAppletSettings::setConfigFile( configFile ); 
    ko2smsAppletSettings::writeConfig(); 
}

void ko2smsApplet::updateConfiguration()
{
    readConfig();
}


extern "C"
{
    KPanelApplet* init( QWidget *parent, const QString& configFile)
    {
        KGlobal::locale()->insertCatalogue("ko2smsapplet");
        return new ko2smsApplet(configFile, KPanelApplet::Normal,
                             KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
                             parent, "ko2smsapplet");
    }
}
